@@ -1,6 +1,6 @@
 logit <- function(x) {  log(x/(1-x))  }
 
-inv.logit <- function(x) {  if(is.numeric(x)) 1/(1+exp(-x)) else stop("x is not numeric!") }
+inv.logit <- function(x) {  1/(1+exp(-x)) }
 
 generateMCMCinits <- function(n.chains, model.pars)
 {
